unit tarbz2;

interface

uses Windows, SysUtils, Classes,
     BZip2, Libtar;

function CompresserTarBz2(ListeFichiersSources: TStrings; FichierDestination: string): bool;
function CompresserTar(ListeFichiersSources: TStrings; FichierDestination: string): bool;
function CompresserBz2(FichierSource, FichierDestination: string): bool;

function DecompresserTarBz2(FichierSource, Repertoire: string): bool;
function DecompresserTar(FichierSource, Repertoire: string): bool;
function DecompresserBz2(FichierSource, FichierDestination: string): bool;

implementation

function CompresserTarBz2(ListeFichiersSources: TStrings; FichierDestination: string): bool;
var DossierTemporaire:array[0..MAX_PATH] of Char;
    DossierTemp: string;
begin
     result:=false;
     if FichierDestination='' then exit;
     GetTempPath(SizeOf(DossierTemporaire), DossierTemporaire);
     DossierTemp := IncludeTrailingPathDelimiter(StrPas(DossierTemporaire));
     if CompresserTar(ListeFichiersSources, DossierTemp+ChangeFileExt(ExtractFileName(FichierDestination),'.tar')) then
     begin
          result:=CompresserBz2(DossierTemp+ChangeFileExt(ExtractFileName(FichierDestination),'.tar'), FichierDestination);
     end;
     DeleteFile(DossierTemp+ChangeFileExt(ExtractFileName(FichierDestination),'.tar'));
end;

function CompresserTar(ListeFichiersSources: TStrings; FichierDestination: string): bool;
var FTarWriter: TTarWriter;
    i: integer;
begin
     try
        FTarWriter := TTarWriter.Create(FichierDestination);
        try
           for i:=0 to ListeFichiersSources.Count-1 do
               FTarWriter.AddFile(ListeFichiersSources[i],ExtractFileName(ListeFichiersSources[i]));
           result:=true;
        except
           result:=false;
        end;
        FTarWriter.Free;
     except
        result:=false;
     end;
end;

function CompresserBz2(FichierSource, FichierDestination: string): bool;
var
  Source, Dest: TStream;
  Comp: TBZCompressionStream;
begin
  Source := TFileStream.Create(FichierSource, fmOpenRead + fmShareDenyWrite);
  try
     Dest := TFileStream.Create(FichierDestination, fmCreate);
     try
        Comp := TBZCompressionStream.Create(bs9, Dest);
        try
           Comp.CopyFrom(Source, 0);
           Comp.Free;
           result:=true;
        except
           result:=false;
        end;
        Dest.Free;
     except
       result:=false;
     end;
     Source.Free;
  except
     result:=false;
  end;
end;

function DecompresserBz2(FichierSource, FichierDestination: string): bool;
const BufferSize = 65536;
var Source, Dest: TStream;
    Count: Integer;
    Decomp: TBZDecompressionStream;
    Buffer: array[0..BufferSize - 1] of Byte;
begin
     try
        Source := TFileStream.Create(FichierSource, fmOpenRead + fmShareDenyWrite);
        try
           Dest := TFileStream.Create(FichierDestination, fmCreate);
           try
              Decomp := TBZDecompressionStream.Create(Source);
              try
                 while True do
                 begin
                      Count := Decomp.Read(Buffer, BufferSize);
                      if Count <> 0 then Dest.WriteBuffer(Buffer, Count) else Break;
                 end;
              finally
                 Decomp.Free;
              end;
           finally
              Dest.Free;
           end;
        finally
          Source.Free;
        end;
        result:=true;
     except
        result:=false;
     end;
end;

function DecompresserTar(FichierSource, Repertoire: string): bool;
var TA        : TTarArchive;
    DirRec    : TTarDirRec;
begin
     try
        TA := TTarArchive.Create(FichierSource);
        try
           while TA.FindNext (DirRec) do
           begin
               if DirRec.FileType=ftDirectory then ForceDirectories(Repertoire+DirRec.Name)
                                              else TA.ReadFile(Repertoire+DirRec.Name);
           end;
        finally
           TA.Free;
        end;
        result:=true;
     except
        result:=false;
     end;
end;

function DecompresserTarBz2(FichierSource, Repertoire: string): bool;
var DossierTemporaire:array[0..MAX_PATH] of Char;
    DossierTemp: string;
begin
     result:=false;
     if Repertoire='' then exit;
     Repertoire:=IncludeTrailingPathDelimiter(Repertoire);
     GetTempPath(SizeOf(DossierTemporaire), DossierTemporaire);
     DossierTemp := IncludeTrailingPathDelimiter(StrPas(DossierTemporaire));

     if DecompresserBz2(FichierSource, DossierTemp+ChangeFileExt(ExtractFileName(FichierSource),'.tar')) then
     begin;
          ForceDirectories(Repertoire);
          result:=DecompresserTar(DossierTemp+ChangeFileExt(ExtractFileName(FichierSource),'.tar'), Repertoire);
     end;
     DeleteFile(DossierTemp+ChangeFileExt(ExtractFileName(FichierSource),'.tar'));
end;


end.
 